@@ -1,6 +1,6 @@
 //===========================================================================
-// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.1/src/DwmIpv6Address.cc 11116 $
-// @(#) $Id: DwmIpv6Address.cc 11116 2020-09-07 22:55:31Z dwm $
+// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.3/src/DwmIpv6Address.cc 11303 $
+// @(#) $Id: DwmIpv6Address.cc 11303 2020-11-25 03:40:42Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2004-2007, 2020
 //  All rights reserved.
@@ -56,7 +56,7 @@ extern "C" {
 #include "DwmSvnTag.hh"
 #include "DwmIpv6Address.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.1/src/DwmIpv6Address.cc 11116 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.3/src/DwmIpv6Address.cc 11303 $");
 
 namespace Dwm {
 
@@ -167,6 +167,24 @@ namespace Dwm {
     return(rc);
   }
 
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Ipv6Address::IsLinkLocal() const
+  {
+    return ((_addr.s6_addr[0] == 0xFE)
+            && ((_addr.s6_addr[1] & 0xC0) == 0x80));
+  }
+
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
+  bool Ipv6Address::IsSiteLocal() const
+  {
+    return ((_addr.s6_addr[0] == 0xFE)
+            && ((_addr.s6_addr[1] & 0xC0) == 0xC0));
+  }
+
   //--------------------------------------------------------------------------
   //!  
   //--------------------------------------------------------------------------
