@@ -1,6 +1,6 @@
 //===========================================================================
-// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.1/src/DwmIpv6Prefix.cc 11117 $
-// @(#) $Id: DwmIpv6Prefix.cc 11117 2020-09-07 22:56:38Z dwm $
+// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.3/src/DwmIpv6Prefix.cc 11306 $
+// @(#) $Id: DwmIpv6Prefix.cc 11306 2020-11-25 21:31:44Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2005-2007, 2020
 //  All rights reserved.
@@ -47,12 +47,13 @@ extern "C" {
 #include <cassert>
 #include <cstdlib>
 #include <cstring>
+#include <regex>
 #include <sstream>
 
 #include "DwmSvnTag.hh"
 #include "DwmIpv6Prefix.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.1/src/DwmIpv6Prefix.cc 11117 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.3/src/DwmIpv6Prefix.cc 11306 $");
 
 namespace Dwm {
 
@@ -62,19 +63,16 @@ namespace Dwm {
   static void
   SetBits(struct in6_addr & addr, uint8_t numBits)
   {
-    uint8_t  byte = 0;
-    if (numBits >= 8) {
-      for ( ; byte < numBits / 8; ++byte)
-        addr.s6_addr[byte] = 0xFF;
+    assert(128 >= numBits);
+    memset(addr.s6_addr, 0, sizeof(addr.s6_addr));
+    uint8_t  *b = addr.s6_addr;
+    while (numBits >= 8) {
+      *b++ = 0xFF;
+      numBits -= 8;
     }
-    addr.s6_addr[byte] = 0;
-    for (uint8_t j = (numBits % 8); j > 0; --j) {
-      addr.s6_addr[byte] |= ((uint8_t)1 << (8 - j));
+    if (numBits) {
+      *b++ = ~(0xFF >> numBits);
     }
-    ++byte;
-    
-    for ( ; byte < 16; ++byte)
-      addr.s6_addr[byte] = 0;
     return;
   }
   
@@ -95,7 +93,7 @@ namespace Dwm {
   //--------------------------------------------------------------------------
   Ipv6Prefix::Ipv6Prefix()
   {
-    memset(&_addr, 0xff, sizeof(_addr));
+    _addr = in6addr_any;
     _length = 128;
   }
 
@@ -104,16 +102,29 @@ namespace Dwm {
   //--------------------------------------------------------------------------
   Ipv6Prefix::Ipv6Prefix(const std::string & prefix)
   {
-    std::string::size_type  addrEnd = prefix.find_first_of('/');
-    if (addrEnd != prefix.npos) {
-      assert(addrEnd < prefix.length());
-      _addr = Ipv6Address(prefix.substr(0, addrEnd));
-      _length = atoi(prefix.substr(addrEnd + 1, prefix.length() - addrEnd).c_str());
-      assert(_length <= 128);
-      MaskBits(_addr, _length);
+    static std::regex
+      rgx("([0-9a-fA-F:]+)([/]([0-9]|[1-9][0-9]|1[01][0-9]|12[0-8])){0,1}",
+          std::regex::ECMAScript | std::regex::optimize);
+
+    std::smatch  sm;
+    if (regex_match(prefix, sm, rgx)) {
+      if (4 == sm.size()) {
+        _addr = Ipv6Address(sm[1].str());
+        if (! sm[3].str().empty()) {
+          _length = atoi(sm[3].str().c_str());
+          MaskBits(_addr, _length);
+        }
+        else {
+          _length = 128;
+        }
+      }
+      else {
+        _addr = Ipv6Address();
+        _length = 128;
+      }
     }
     else {
-      _addr = Ipv6Address(prefix);
+      _addr = Ipv6Address();
       _length = 128;
     }
   }
@@ -124,6 +135,7 @@ namespace Dwm {
   Ipv6Prefix::Ipv6Prefix(const Ipv6Address & network, uint8_t maskLength)
   {
     _addr = network;
+    assert(128 >= maskLength);
     _length = maskLength;
     MaskBits(_addr, _length);
   }
@@ -145,6 +157,9 @@ namespace Dwm {
     return(_addr);
   }
   
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
   Ipv6Address Ipv6Prefix::Netmask() const
   {
     in6_addr  addr;
@@ -152,11 +167,17 @@ namespace Dwm {
     return(addr);
   }
   
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
   uint8_t Ipv6Prefix::MaskLength() const
   {
     return(_length);
   }
   
+  //--------------------------------------------------------------------------
+  //!  
+  //--------------------------------------------------------------------------
   bool Ipv6Prefix::Set(const Ipv6Address & network, uint8_t maskLength)
   {
     _addr = network;
