@@ -1,6 +1,6 @@
 //===========================================================================
-// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.1/tests/TestIpv6Prefix.cc 8389 $
-// @(#) $Id: TestIpv6Prefix.cc 8389 2016-04-17 04:31:36Z dwm $
+// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.3/tests/TestIpv6Prefix.cc 11305 $
+// @(#) $Id: TestIpv6Prefix.cc 11305 2020-11-25 21:27:11Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2005-2007
 //  All rights reserved.
@@ -48,12 +48,14 @@ extern "C" {
 static const char storeFile[] = "/tmp/Ipv6Prefix.store";
 
 #include <fstream>
+#include <tuple>
+#include <vector>
 
 #include "DwmIpv6Prefix.hh"
 #include "DwmSvnTag.hh"
 #include "DwmUnitAssert.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.1/tests/TestIpv6Prefix.cc 8389 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.3/tests/TestIpv6Prefix.cc 11305 $");
 
 using namespace std;
 using namespace Dwm;
@@ -110,6 +112,9 @@ void TestOperators()
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
 void TestIO()
 {
   Ipv6Prefix  prefix1("4444:8002:5:2::2/64");
@@ -176,6 +181,45 @@ void TestIO()
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static void TestStringConstructor()
+{
+  const vector<tuple<string,string,uint8_t>>  goodPrefixes = {
+    { "4444:8002:5:2::2/64",  "ffff:ffff:ffff:ffff::", 64 },
+    { "4444:8001:5:2::2/30",  "ffff:fffc::", 30 },
+    { "4444:8001:5:2::3",     "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", 128 },
+    { "4444:8001:5:2::4/128", "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", 128 },
+    { "4444:8001:5:2::5/0",   "0::", 0 }
+  };
+
+  for (const auto & t : goodPrefixes) {
+    Ipv6Prefix  pfx(get<0>(t));
+    UnitAssert(pfx.MaskLength() == get<2>(t));
+    UnitAssert(pfx.Netmask() == Ipv6Address(get<1>(t)));
+  }
+
+  const vector<tuple<string,in6_addr,string,uint8_t>>  badPrefixes = {
+    { "4444:8002:5:2::2/129", in6addr_any,
+      "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", 128 },
+    { "4444:8001:5:2::2/a",   in6addr_any,
+      "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", 128 },
+    { "4444:80G1:5:2::3",     in6addr_any,
+      "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", 128 },
+    { "4444:8001:5:2::Q/128", in6addr_any,
+      "ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff", 128 }
+  };
+  for (const auto & t : badPrefixes) {
+    Ipv6Prefix  pfx(get<0>(t));
+    UnitAssert(pfx.MaskLength() == get<3>(t));
+    UnitAssert(pfx.Netmask() == Ipv6Address(get<2>(t)));
+    UnitAssert(pfx.Network() == Ipv6Address(in6addr_any));
+  }
+  
+  return;
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -184,6 +228,7 @@ int main(int argc, char *argv[])
   TestNetmask();
   TestOperators();
   TestIO();
+  TestStringConstructor();
   
   if (Assertions::Total().Failed()) {
     Assertions::Print(cerr, true);
