@@ -1,6 +1,6 @@
 //===========================================================================
-// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.1/tests/TestThreadQueue.cc 10193 $
-// @(#) $Id: TestThreadQueue.cc 10193 2019-11-24 01:19:18Z dwm $
+// @(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.3/tests/TestThreadQueue.cc 11256 $
+// @(#) $Id: TestThreadQueue.cc 11256 2020-11-15 22:52:10Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2016
 //  All rights reserved.
@@ -53,7 +53,7 @@ extern "C" {
 #include "DwmUnitAssert.hh"
 #include "DwmThreadQueue.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.1/tests/TestThreadQueue.cc 10193 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/libDwm/tags/libDwm-0.8.3/tests/TestThreadQueue.cc 11256 $");
 
 using namespace std;
 using namespace Dwm;
@@ -69,6 +69,17 @@ void PushPopProducer(Thread::Queue<int> & tq)
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+void PushPopFrontProducer(Thread::Queue<int> & tq)
+{
+  for (int i = 0; i < 1000; ++i) {
+    UnitAssert(tq.PushFront(i));
+  }
+  return;
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -90,6 +101,27 @@ void PushPopConsumer(Thread::Queue<int> & tq)
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+void PushPopBackConsumer(Thread::Queue<int> & tq)
+{
+  int  i = 0;
+  int  value;
+  //  This is a while() loop because PopBack() might not have anything
+  //  to consume if this thread gets more time than the producer thread.
+  //  We only increment the loop termination condition when we actually
+  //  consumed a member of the queue.
+  while (i < 1000) {
+    if (tq.PopBack(value)) {
+      UnitAssert(value == i);
+      ++i;
+    }
+  }
+  UnitAssert(i == 1000);
+  return;
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -100,6 +132,10 @@ void TestPushPop()
   thread  consumerThread(PushPopConsumer, std::ref(tq));
   producerThread.join();
   consumerThread.join();
+  thread  frontProducerThread(PushPopFrontProducer, std::ref(tq));
+  thread  backConsumerThread(PushPopBackConsumer, std::ref(tq));
+  frontProducerThread.join();
+  backConsumerThread.join();
 }
 
 //----------------------------------------------------------------------------
@@ -222,7 +258,9 @@ void BulkPopFn(Thread::Queue<uint64_t> * queue, const string & name)
   deque<uint64_t>  entries;
   while (processed < 1000000) {
     processed += queue->Swap(entries);
+    entries.clear();
   }
+  UnitAssert(1000000 == processed);
   return;
 }
 
@@ -241,6 +279,35 @@ void BulkPushFn(Thread::Queue<uint64_t> * queue, const string & name)
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+void BulkPushFrontFn(Thread::Queue<uint64_t> * queue, const string & name)
+{
+  uint64_t  pushed = 0;
+  vector<uint64_t>  entries(1000);
+  while (pushed < 1000000) {
+    if (queue->PushFront(entries.begin(), entries.end())) {
+      pushed += entries.size();
+    }
+  }
+  return;
+}
+
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+void TestBulkPushFront()
+{
+  Thread::Queue<uint64_t>  tq;
+  thread  pushThread(BulkPushFrontFn, &tq, "bulk_push_front");
+  thread  popThread(BulkPopFn, &tq, "pop");
+  pushThread.join();
+  popThread.join();
+  UnitAssert(tq.Empty());
+  return;
+}
+                    
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -324,7 +391,8 @@ int main(int argc, char *argv[])
   TestPushPop();
   TestWaitForNotEmpty();
   TestConditionTimedWait();
-
+  TestBulkPushFront();
+  
   if (optargs.Get<bool>('p')) {
     Dwm::SysLogger::Open("TestThreadQueue", LOG_PERROR, LOG_USER);
     TestSimplePerformance();
